@@ -27,13 +27,13 @@
  * General driver to allow command-line fuzzer (i.e. afl) to
  * exercise the libFuzzer entrypoint.
  */
+#include <stdio.h>
 
 #include <sys/types.h>
 #include <fcntl.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef WIN32
+#ifdef _WIN32
 #  include <io.h>
 #else
 #  include <unistd.h>
@@ -75,7 +75,11 @@ int main(int argc, char *argv[])
   if (argc == 1) {
     int count = 0;
     while (KEEP_FUZZING(count)) {
+#ifndef STDIN_FILENO
       ProcessFile(fileno(stdin));
+#else
+      ProcessFile(STDIN_FILENO);
+#endif
       count++;
     }
   } else {
